import os
import tempfile
import uuid

import gradio as gr
from ultoical.ul_handler import ULHandler
from datetime import datetime
from ultoical.ical_handler import ICalHandler
import zipfile

class ULToICALInterface:
  def login_with_password(self, username: str, password: str):
    if self.handler.login_with_password(username, password):
      gr.Info("Login successful / Connection reussie")
    else:
      gr.Warning("There was an error connecting into the account / Une erreur est apparu lors de la connexion")
  def login_with_token(self, token: str):
    self.handler.set_token(token)

  def get_timetables(self, start: float, end: float):
    timetables = self.handler.get_timetables(datetime.fromtimestamp(start), datetime.fromtimestamp(end))
    ret = []
    for timetabl in timetables["plannings"]:
        ret.append(timetabl["label"])
    return gr.CheckboxGroup(choices=ret)

  def makeical(self, timetables):
    files = []
    for timetable in self.handler.get_cached_timetable()["plannings"]:
      if timetable["label"] in timetables:
        ical = ICalHandler(timetable["events"]).make_ical()
        files.append(ical)
        self.tmps.append(ical)
    if len(files) == 1:
      return gr.DownloadButton(value=files[0])
    else:
      file = tempfile.gettempdir() + "/" + str(uuid.uuid4()) + ".zip"
      _zip = zipfile.ZipFile(file, "w", zipfile.ZIP_DEFLATED)
      for file in files:
        _zip.write(file)
      _zip.close()
      self.tmps.append(_zip.filename)
      return gr.DownloadButton(value=_zip.filename)

  def remove_tmps(self):
    for tmp in self.tmps:
        os.remove(tmp)

  def __init__(self):
    self.tmps = []
    self.timetable = None
    self.handler = ULHandler()
    with gr.Blocks() as app:
      gr.Markdown("# ULToICAL")
      with gr.Tab("Login using username and password / Connection via nom d'utilisateur et mot de passe"):
        gr.Markdown(
          "## Log into your account using your username and password")
        gr.Markdown("Se connecter au compte en utilisant un nom d'utilisateur et un mot de passe")
        with gr.Row(equal_height=True):
          self.username = gr.Textbox(label="Your username / Nom d'utilisateur", value="")
          self.password = gr.Textbox(label="Your password / Mot de passe", value="", type="password")
          self.login_button = gr.Button("Login / Connexion")
        self.login_button.click(fn=self.login_with_password, inputs=[self.username, self.password])
      with gr.Tab("Login using token / Connection par jeton"):
        gr.Markdown("## Log into your account using token (More info [here](https://github.com/totorocodesoften/ultoical/readme.md))")
        gr.Markdown("Se connecter au compte en utilisant un jeton (Plus d'informations [ici](https://github.com/totorocodesoften/ultoical/readme.fr.md)")
        with gr.Row(equal_height=True):
          self.token = gr.Textbox(label="Your token / Votre jeton", value="")
          self.login_button = gr.Button("Login / Connexion")
        self.login_button.click(fn=self.login_with_token, inputs=[self.token])
      gr.Markdown("## Choose dates")
      gr.Markdown("Choisissez les dates")
      with gr.Row(equal_height=True) as self.period:
        with gr.Column():
          self.start = gr.DateTime(label="Start of the period / Debut de la periode", include_time=False)
        with gr.Column():
            self.end = gr.DateTime(label="End of the period / Fin de la periode", include_time=False)
        self.get_timetables_button = gr.Button("Get timetables / Obtenir les emplois du temps")
      gr.Markdown("## Timetables")
      gr.Markdown("Emploi du temps")
      self.timetable = gr.CheckboxGroup(label="Timetable / Emplois du temps", info="If none are there the connection failed / Si aucun n'est apparu il y a eu un probleme de connexion")
      self.get_timetables_button.click(fn=self.get_timetables, inputs=[self.start, self.end], outputs=[self.timetable])
      gr.Markdown("## Download")
      gr.Markdown("Telechargement")
      dwnbtn = gr.DownloadButton()
      self.timetable.input(fn=self.makeical, inputs=[self.timetable], outputs=[dwnbtn])
      dwnbtn.click(fn=self.remove_tmps)
    app.launch()



import uuid
from datetime import datetime
import tempfile

def truncate(string, width):
    if len(string) > width:
        string = string[:width-3] + '...'
    return string

class ICalHandler:

    def __init__(self, content):
        self.content = content

    def make_ical(self):
        file = tempfile.gettempdir() + "/" + str(uuid.uuid4()) + ".ics"
        f = open(file, "w")
        f.write("""BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//github.com//totorocodesoften//ultoical
CALSCALE:GREGORIAN
BEGIN:VTIMEZONE
TZID:Europe/Paris
LAST-MODIFIED:20240422T053451Z
TZURL:https://www.tzurl.org/zoneinfo-outlook/Europe/Paris
X-LIC-LOCATION:Europe/Paris
BEGIN:DAYLIGHT
TZNAME:CEST
TZOFFSETFROM:+0100
TZOFFSETTO:+0200
DTSTART:19700329T020000
RRULE:FREQ=YEARLY;BYMONTH=3;BYDAY=-1SU
END:DAYLIGHT
BEGIN:STANDARD
TZNAME:CET
TZOFFSETFROM:+0200
TZOFFSETTO:+0100
DTSTART:19701025T030000
RRULE:FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU
END:STANDARD
END:VTIMEZONE
""".replace("\n", "\r\n"))
        dtstamp = datetime.now().strftime("%Y%m%dT%H%M%SZ")
        for event in self.content:
            start = event["startDateTime"].replace("+01:00", "").replace(":", "").replace("-", "")
            end = event["endDateTime"].replace("+01:00", "").replace(":", "").replace("-", "")
            teacher = event["teachers"][0]["displayname"] if len(event["teachers"]) > 0 else ""
            room = event["rooms"][0] if len(event["rooms"]) > 0 else {"label": "", "building": ""}
            desc = "{0:<10s}".format(truncate(event["course"]["label"] + " " + room["label"] + " " + teacher, 60))

            f.write(f"""BEGIN:VEVENT
DTSTAMP:{dtstamp}
UID:{uuid.uuid4()}@ultoical.totorocodesoften
DTSTART;TZID=Europe/Paris:{start}
DTEND;TZID=Europe/Paris:{end}
SUMMARY:{"{0:<10s}".format(truncate(event["course"]["label"], 65))}
URL:{event["course"]["url"]}
DESCRIPTION:{desc}
LOCATION:{room["label"]}, {room["building"]}
CLASS:PRIVATE
BEGIN:VALARM
ACTION:DISPLAY
DESCRIPTION:{"{0:<10s}".format(truncate(event["course"]["label"], 30))} is soon, go to {room["label"]}
TRIGGER:-PT5M
END:VALARM
END:VEVENT
""".replace("\n", "\r\n"))
        f.write("END:VCALENDAR")
        f.close()
        # get the path from the oppened file
        return file


from importlib.metadata import entry_points

from setuptools import setup, find_packages
setup(
    name="ultoical",
    version="1.1.2",
    author="totorocodesoften",
    author_email="totorocodesoften@github.com",
    description="A web app allowing you to create an ical file from your Universite de Lorraine's timetable",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/totorocodesoften/ultoical',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.10',
    install_requires=[
        "setuptools~=75.8.0",
        "gradio~=5.13.1",
        "requests~=2.32.3"
    ],
    extras_require={
    },
    entry_points={
        'console_scripts': [
            'ultoical = ultoical.__main__:main'
        ]
    },
    include_package_data=True,
    package_data={
        "": [
            "*.txt", "*.rst"
        ],
        "ultoical": [
            "data/*.dat"
        ]
    }
)


import datetime

import requests
from datetime import datetime

class ULHandler:
    def __init__(self):
        self.token = None
        self.timetables = None

    def set_token(self, token:str):
        self.token = token

    def get_cached_timetable(self):
        return self.timetables

    def get_timetables(self, start:datetime, end:datetime):
        if self.timetables is None:
            headers = {
                'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:134.0) Gecko/20100101 Firefox/134.0',
                'Accept': 'application/json, text/plain, */*',
                'Accept-Language': 'en-US,en;q=0.5',
                'Content-Type': 'application/json',
                'Origin': 'https://mobile.univ-lorraine.fr',
                'Connection': 'keep-alive',
                'Referer': 'https://mobile.univ-lorraine.fr/',
                'Sec-Fetch-Dest': 'empty',
                'Sec-Fetch-Mode': 'cors',
                'Sec-Fetch-Site': 'same-site',
                'Priority': 'u=0',
            }

            json_data = {
                'authToken': self.token,
                'startDate': start.strftime("%Y-%m-%d"),
                'endDate': end.strftime("%Y-%m-%d"),
                'asUser': None,
            }

            response = requests.post('https://mobile-back.univ-lorraine.fr/schedule', headers=headers, json=json_data)
            self.timetables = response.json()
        return self.timetables

    def login_with_password(self, username, password):
        headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:134.0) Gecko/20100101 Firefox/134.0',
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            # 'Accept-Encoding': 'gzip, deflate, br, zstd',
            'Content-Type': 'application/json',
            'Origin': 'https://mobile.univ-lorraine.fr',
            'DNT': '1',
            'Sec-GPC': '1',
            'Connection': 'keep-alive',
            'Referer': 'https://mobile.univ-lorraine.fr/',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
            'Priority': 'u=0',
            # Requests doesn't support trailers
            # 'TE': 'trailers',
        }

        json_data = {
            'username': username,
            'password': password,
        }

        response = requests.post('https://mobile-back.univ-lorraine.fr/auth', headers=headers, json=json_data)
        if "authToken" in response.json():
            self.token = response.json()["authToken"]
            return True
        else:
            return False